Option Explicit On
Option Strict On

Imports System
Imports System.Query
Imports System.Data
Imports System.Data.SqlClient
Imports System.Xml
Imports System.Xml.XLinq
Imports System.Text
'Following is required if you substitute BindingList(Of T) for List(Of T)
'Imports System.ComponentModel

Module Module1

    '***************************************************************************
    'Workarounds for lack of DLinq support in initial VB LINQ Technology Preview
    'The VB Tech Preview doesn't install the DLinq library (the C# TP does)
    'This project requires a connection to an SQL Server instance that includes
    'the Northwind sample database. Change strConn to reflect your installation
    '
    'The examples also demonstrate unsupported VB 9.0 query operator syntax
    '***************************************************************************

    'Entity classes
    Private Customers As New List(Of Northwind.Customer)
    Private Orders As New List(Of Northwind.[Order])
    Private Products As New List(Of Northwind.Product)

    Private strConn As String = "Server=localhost;Database=Northwind;Integrated Security=SSPI"

    Sub Main()
        'Console display for Northwind objects
        Console.WriteLine("Northwind Customers in USA" + vbCrLf)
        Console.WriteLine("ID    Company Name                             Contact Name")
        Console.WriteLine("----- ---------------------------------------- -----------------------------")
        GetCustomers()

        'Customers in USA
        Dim CustsUSA = Select c From c In Customers Where c.Country = "USA"
        Dim ieCusts As IEnumerator(Of Northwind.Customer) = CustsUSA.GetEnumerator

        'Queries aren't executed until calling For Each or another statement to return values
        For Each CustUSA In CustsUSA
            'Format right-justified
            Console.WriteLine("{0, -6}{1, -41}{2, -31}", CustUSA.CustomerID, CustUSA.CompanyName, CustUSA.ContactName)
        Next

        'Count() isn't operable in initial release
        'Dim CountUSA = Select Count(c) From c In Customers Where c.Country = "USA"

        'Orders shipped to USA (also demonstrates Console.WriteLine formatting)
        Console.WriteLine(vbCrLf + "Northwind Orders for Shipment to USA" + vbCrLf)
        Console.WriteLine("OrderID CustID EmpID  OrderDate  Req'dDate   ShipDate ShipVia Freight")
        Console.WriteLine("-----   -----  --    ---------- ---------- ---------- --      -------")
        GetOrders()

        'Joins aren't operable in the initial release:
        'Dim OrdersUSA = Select c.CustomerID, o.OrderID From c In Customers, o In Orders Where o.CustomerID = c.CustomerID And c.Country = "USA"

        'Comma-separated Select column lists aren't operable in initial release:
        'Dim OrdersUSA = Select o.OrderID, o.CustomerID, o.EmployeeID From o In Orders Where o.ShipCountry = "USA" Order By o.CustomerID, o.OrderID

        'Order By column1 Asc, column2 Desc isn't supported in the initial release

        Dim OrdersUSA = Select o From o In Orders Where o.ShipCountry = "USA" Order By o.CustomerID, o.OrderID
        For Each OrderUSA In OrdersUSA
            Console.WriteLine("{0, -8}{1, -7}{2, 2}{3, 14:d}{4, 11:d}{5, 11:d}{6, 3}{7, 13:c2}", OrderUSA.OrderID, OrderUSA.CustomerID, OrderUSA.EmployeeID, OrderUSA.OrderDate, OrderUSA.RequiredDate, OrderUSA.ShippedDate, OrderUSA.ShipVia, OrderUSA.Freight)
        Next

        'Products not discontinued
        Console.WriteLine(vbCrLf + "Northwind Products Not Discontinued" + vbCrLf)
        Console.WriteLine("ID ProductName                              VendID CatID QuantityPerUnit      ")
        Console.WriteLine("-- ---------------------------------------- --     --    -------------------- ")
        GetProducts()

        Dim ProdsCurrent = Select p From p In Products Where p.Discontinued = False Order By p.ProductName
        For Each Prod In ProdsCurrent
            'Write only the first five fields 
            Console.WriteLine("{0, 2} {1, -41}{2, 2}     {3,2}    {4, -21}", Prod.ProductID, Prod.ProductName, Prod.SupplierID, Prod.CategoryID, Prod.QuantityPerUnit, Prod.UnitPrice, Prod.UnitsInStock, Prod.UnitsOnOrder, Prod.ReorderLevel, Prod.Discontinued)
        Next

        'Demo of query expression service that's similar to the Jet Expression service
        Console.WriteLine(vbCrLf + "Query 'Expression Service' (similar to the Jet Expression Service)" + vbCrLf)
        Dim FixFreight = Select decFixFreight(o.ShipVia, o.Freight) From o In Orders Where o.ShipCountry = "Canada"
        For Each Freight In FixFreight
            Console.WriteLine("Adjusted Freight: {0, 10}", Freight)
        Next

        'VBTeam-related samples (uncomment to display)
        'SimpleListOfCustomers()
        'PrimeNumbers()

        Console.WriteLine(vbCrLf + "Press any key to exit ...")
        Console.ReadLine()

        'Uncomment below and use statement completion to view Standard Query Operator functions
        'System.Query.Sequence
    End Sub

    Public Function decFixFreight(ByVal intShipVia As Integer, ByVal decFreight As Nullable(Of Decimal)) As Nullable(Of Decimal)
        'Adjust Freight charge by 10, 20 or 30% depending on shipper
        Dim decNewFreight As Nullable(Of Decimal)
        If decFreight.HasValue Then
            Select Case intShipVia
                Case 1
                    decNewFreight = 1.1D * CDec(decFreight)
                Case 2
                    decNewFreight = 1.2D * CDec(decFreight)
                Case 3
                    decNewFreight = 1.3D * CDec(decFreight)
                Case Else
                    decNewFreight = CDec(decFreight)
            End Select
            Return decNewFreight
        Else
            Return Nothing
        End If
    End Function

    Public Sub GetCustomers()
        'Following emulates DataContext to generate up to 91 Northwind Customer entity class instances
        Dim cnNwind As New SqlConnection(strConn)
        Dim cmCusts As New SqlCommand("SELECT * FROM Customers", cnNwind)
        Dim objCust() As Object
        Try
            cnNwind.Open()
            Dim sdrCusts As SqlDataReader = cmCusts.ExecuteReader
            With sdrCusts
                If .HasRows Then
                    ReDim objCust(.FieldCount - 1)
                    Dim objList As Northwind.Customer
                    Dim intCol As Integer
                    While .Read
                        .GetValues(objCust)
                        objList = New Northwind.Customer
                        For intCol = 0 To objCust.Length - 1
                            Select Case intCol
                                Case 0
                                    objList.CustomerID = objCust(intCol).ToString
                                Case 1
                                    objList.CompanyName = objCust(intCol).ToString
                                Case 2
                                    objList.ContactName = objCust(intCol).ToString
                                Case 3
                                    objList.ContactTitle = objCust(intCol).ToString
                                Case 4
                                    objList.Address = objCust(intCol).ToString
                                Case 5
                                    objList.City = objCust(intCol).ToString
                                Case 6
                                    objList.Region = objCust(intCol).ToString
                                Case 7
                                    objList.PostalCode = objCust(intCol).ToString
                                Case 8
                                    objList.Country = objCust(intCol).ToString
                                Case 9
                                    objList.Phone = objCust(intCol).ToString
                                Case 10
                                    objList.Fax = objCust(intCol).ToString
                            End Select
                        Next intCol
                        Customers.Add(objList)
                    End While
                End If
                .Close()
            End With
            'Dim intCount As Integer = Customers.Count
        Catch exc As Exception
            MsgBox(exc.Message)
        Finally
            cnNwind.Close()
            cnNwind.Dispose()
            cmCusts.Dispose()
        End Try
    End Sub

    Public Sub GetOrders()
        'Following emulates DataContext to generate up to 830 Northwind Order entity class instances
        Dim cnNwind As New SqlConnection(strConn)
        Dim cmOrders As New SqlCommand("SELECT OrderID, CustomerID, EmployeeID, OrderDate, RequiredDate, ShippedDate, ShipVia, Freight, ShipCountry FROM Orders", cnNwind)
        Dim objOrder() As Object
        Try
            cnNwind.Open()
            Dim sdrOrders As SqlDataReader = cmOrders.ExecuteReader
            With sdrOrders
                If .HasRows Then
                    ReDim objOrder(.FieldCount - 1)
                    Dim objList As Northwind.[Order]
                    Dim intCol As Integer
                    While .Read
                        .GetValues(objOrder)
                        objList = New Northwind.[Order]
                        For intCol = 0 To objOrder.Length - 1
                            Select Case intCol
                                Case 0
                                    objList.OrderID = CType(objOrder(intCol), Integer)
                                Case 1
                                    objList.CustomerID = objOrder(intCol).ToString
                                Case 2
                                    objList.EmployeeID = CType(objOrder(intCol), Integer)
                                Case 3
                                    objList.OrderDate = CType(objOrder(intCol), Date)
                                Case 4
                                    objList.RequiredDate = CType(objOrder(intCol), Date)
                                Case 5
                                    If IsDBNull(objOrder(intCol)) Then
                                        objList.ShippedDate = Nothing
                                    Else
                                        objList.ShippedDate = CType(objOrder(intCol), Date)
                                    End If
                                Case 6
                                    objList.ShipVia = CType(objOrder(intCol), Integer)
                                Case 7
                                    objList.Freight = CType(objOrder(intCol), Decimal)
                                Case 8
                                    objList.ShipCountry = objOrder(intCol).ToString
                            End Select
                        Next intCol
                        Orders.Add(objList)
                    End While
                End If
                .Close()
            End With
            'Dim intCount As Integer = Orders.Count
        Catch exc As Exception
            MsgBox(exc.Message)
        Finally
            cnNwind.Close()
            cnNwind.Dispose()
            cmOrders.Dispose()
        End Try
    End Sub

    Public Sub GetProducts()
        'Following emulates DataContext to generate up to 77 Northwind Product entity class instances
        Dim cnNwind As New SqlConnection(strConn)
        Dim cmProds As New SqlCommand("SELECT * FROM Products", cnNwind)
        Dim objProd() As Object
        Try
            cnNwind.Open()
            Dim sdrProds As SqlDataReader = cmProds.ExecuteReader
            With sdrProds
                If .HasRows Then
                    ReDim objProd(.FieldCount - 1)
                    Dim objList As Northwind.Product
                    Dim intCol As Integer
                    While .Read
                        .GetValues(objProd)
                        objList = New Northwind.Product
                        For intCol = 0 To objProd.Length - 1
                            Select Case intCol
                                Case 0
                                    objList.ProductID = CType(objProd(intCol), Integer)
                                Case 1
                                    objList.ProductName = objProd(intCol).ToString
                                Case 2
                                    objList.SupplierID = CType(objProd(intCol), Integer)
                                Case 3
                                    objList.CategoryID = CType(objProd(intCol), Integer)
                                Case 4
                                    objList.QuantityPerUnit = objProd(intCol).ToString
                                Case 5
                                    objList.UnitPrice = CType(objProd(intCol), Decimal)
                                Case 6
                                    objList.UnitsInStock = CType(objProd(intCol), Short)
                                Case 7
                                    objList.UnitsOnOrder = CType(objProd(intCol), Short)
                                Case 8
                                    objList.ReorderLevel = CType(objProd(intCol), Short)
                                Case 9
                                    objList.Discontinued = CType(objProd(intCol), Boolean)
                            End Select
                        Next intCol
                        Products.Add(objList)
                    End While
                End If
                .Close()
            End With
            'Dim intCount As Integer = Products.Count
        Catch exc As Exception
            MsgBox(exc.Message)
        Finally
            cnNwind.Close()
            cnNwind.Dispose()
            cmProds.Dispose()
        End Try
    End Sub

    '**********************************************
    'Simple procedures from or based on VBTeam code
    '**********************************************

    Public Sub SimpleListOfCustomers()
        'Example based on code from Amanda Silver's VBTeam blog item
        'http://blogs.msdn.com/vbteam/archive/2005/09/13/464893.aspx

        Dim TestCusts As New List(Of TestCust)

        Dim Cust1 As TestCust = {Name := "Nancy", Address := "123 Main St."}
        TestCusts.Add(Cust1)

        'With-style syntax doesn't work in initial bits
        'Dim Cust3 As TestCust = {.Name = "Joel", .Address = "234 Elm St."}
        Dim Cust2 As TestCust = {Name := "Joel", Address := "234 Elm St."}
        TestCusts.Add(Cust2)

        Cust2 = New TestCust
        Cust2.Name = "Billy"
        Cust2.Address = "456 Maple St."
        TestCusts.Add(Cust2)

        Console.WriteLine(vbCrLf + "Simple Customer List (from VBTeam)" + vbCrLf)
        For Each Cust In TestCusts
            Console.WriteLine("{0, -10}{1, -10}", Cust.Name, Cust.Address)
        Next

        Console.WriteLine(vbCrLf + "Select from Simple Customer List (from VBTeam)" + vbCrLf)
        Dim custJoel = Select c From c In TestCusts Where c.Name = "Joel"

        For Each Cust In custJoel
            Console.WriteLine("{0, -10}{1, -10}", Cust.Name, Cust.Address)
        Next
        Console.WriteLine()
    End Sub

    Public Class TestCust
        'Simple Customer entity class
        Public Name As String
        Public Address As String
    End Class

    Public Sub PrimeNumbers()
        'Example of code from Amanda Silver's VBTeam blog item
        'http://blogs.msdn.com/vbteam/archive/2005/09/13/464893.aspx

        'Note that 1 is NOT a prime number
        'Dim primes = {1, 2, 3, 5, 7, 9, 11, 13}

        Dim primes = {2, 3, 5, 7, 9, 11, 13}
        Dim primesOverFive = Select p From p In primes Where p > 5

        Console.WriteLine(vbCrLf + "Prime Numbers > 5 (from VBTeam)" + vbCrLf)
        For Each test In primesOverFive
            Console.WriteLine(test)
        Next
        Console.WriteLine()
    End Sub

    Public Sub ObjectInitializers()
        'Examples based on code from Amanda Silver's VBTeam blog item (no console output)
        'http://blogs.msdn.com/vbteam/archive/2005/09/13/464893.aspx
        Dim int = 1
        Dim doub = 1.2
        Dim str = "Hello"
        'Simple array initializer
        Dim intArr = {1, 2, 3}
        'Two-dimension array initializer
        Dim strArr = {{"Fred", "Barney"}, {"George", "Judy"}}
        Dim strBuilderArr = {New StringBuilder(), New StringBuilder()}
        'Object array (data types differ)
        Dim objArr = {"Fred", 10}
    End Sub

End Module
