Option Explicit On
Option Strict On
Imports System.Math

Public Class XLinq

    Private Sub XLinq_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Dim strMsg As String = "The XLinqTestProjectVB project expands on the examples included in Erik Meijer's " + _
        """XLinq: XML Programming Refactored (The Return Of The Monoids)"" presentation " + _
        "(http://www.idealliance.org/xmlusa/05/call/xmlpapers/63.1015/.63.html) at the " + _
        "XML 2005 Conference to be held in Atlanta on November 14 - 18, 2005. Following is the " + _
        "abstract of the presentation:" + vbCrLf + vbCrLf
        strMsg += """We propose to refactor special-purpose XML centric languages into their essential " + _
        "constituents. Instead of creating specialized languages for querying XML, general purpose " + _
        "languages should be extended with query capabilities based on monoids, allowing programmers " + _
        "to query any form of data, using a broad notion of 'collection'." + vbCrLf + vbCrLf
        strMsg += "Instead of the DOM, we need to create a lightweight, rational, and simple API for " + _
        "manipulating XML that is designed specifically to play well with this general query infrastructure. " + _
        "In particular this implies that the API needs to support creation of XML that is node centric " + _
        "as opposed to document centric, and expression-oriented as opposed to the imperative " + _
        "statement-oriented style of the DOM.""" + vbCrLf + vbCrLf
        strMsg += "This project demonstrates composing well-formed XML documents from Northwind sample data by the use of: " + vbCrLf + vbCrLf + _
        "1. Expressions that use the XLinq API's XDocument, XDeclaration, XComment and XElement classes." + vbCrLf + vbCrLf + _
        "2. Expressions that use the XLinq API's XElement and add elements with calculated valuse in a specified position in the document." + vbCrLf + vbCrLf + _
        "3. Visual Basic-only XML Literals created by copying and pasting XML content as VB 9.0 code."
        txtXML.Text = strMsg
    End Sub

    Private Sub btnXDocument_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnXDocument.Click
        SalesOrderDocument()
    End Sub

    Private Sub btnXElement_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnXElement.Click
        SalesOrderExpression()
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        SalesOrderLiteral()
    End Sub

    Private Sub SalesOrderDocument()
        'Create a static SalesOrders XDocument from XLinq expressions

        'Define a static BillTo group
        Dim expBillTo = _
        New XElement("BillTo", _
        New XAttribute("CustomerID", "RATTC"), _
        New XElement("CompanyName", "Rattlesnake Canyon Grocery"), _
        New XElement("ContactName", "Paula Wilson"), _
        New XElement("ContactTitle", "Assistant Sales Representative"), _
        New XElement("Address", "2817 Milton Dr."), _
        New XElement("City", "Albuquerque"), _
        New XElement("Region", "NM"), _
        New XElement("PostalCode", "87110"), _
        New XElement("Country", "USA"), _
        New XElement("Phone", "(505) 555-5939"), _
        New XElement("Fax", "(505) 555-3620"))

        'Define a static ShipTo group
        Dim expShipTo = _
        New XElement("ShipTo", _
        New XElement("ShipName", "Rattlesnake Canyon Grocery"), _
        New XElement("ShipAddress", "2821 Milton Dr."), _
        New XElement("ShipCity", "Albuquerque"), _
        New XElement("ShipRegion", "NM"), _
        New XElement("ShipPostalCode", "87110"), _
        New XElement("ShipCountry", "USA"))

        'Define three static LineItem groups
        Dim expLineItem1 = _
        New XElement("LineItem", _
        New XAttribute("OrderID", 10262), _
        New XElement("ProductID", 5), _
        New XElement("UnitPrice", 17.0), _
        New XElement("Quantity", 12), _
        New XElement("Discount", 0.2))

        Dim expLineItem2 = _
        New XElement("LineItem", _
        New XAttribute("OrderID", 10262), _
        New XElement("ProductID", 7), _
        New XElement("UnitPrice", 24.0), _
        New XElement("Quantity", 15), _
        New XElement("Discount", 0.0))

        Dim expLineItem3 = _
        New XElement("LineItem", _
        New XElement("Quantity", 2), _
        New XAttribute("OrderID", 10262), _
        New XElement("ProductID", 56), _
        New XElement("UnitPrice", 30.4), _
        New XElement("Discount", 0.0))

        'Create a SalesOrder document
        'Note that XDeclaration doesn't work in the initial Tech Preview
        'and implementation XDocumentType isn't finished (per XLinq Overview.doc)
        Dim expOrder = _
        New XDocument( _
        New XDeclaration("1.0", "UTF-8", "yes"), _
        New XDocumentType(), _
        New XComment("This is a demonstration of parenting a complex XElement"), _
        New XProcessingInstruction("XLinkTestProjectVB", "ProcessingInstructions"), _
        New XElement("SalesOrder", _
        New XAttribute("OrderID", 10262), _
        New XAttribute("EmployeeID", 8), _
        New XAttribute("ShipVia", 3), _
        expBillTo, _
        New XElement("OrderDate", "1996-7-22"), _
        New XElement("RequiredDate", "1996-8-19"), _
        New XElement("ShipDate", "1996-7-25"), _
        New XElement("Freight", 48.29), _
        expShipTo, _
        New XElement("LineItems", _
        expLineItem1, expLineItem2, expLineItem3)))

        'Display the formatted document
        txtXML.Text = expOrder.ToString
    End Sub

    Private Sub SalesOrderExpression()
        'Create a static SalesOrders XElement from XLinq expressions

        'Define a static BillTo group
        Dim expBillTo = _
        New XElement("BillTo", _
        New XAttribute("CustomerID", "RATTC"), _
        New XElement("CompanyName", "Rattlesnake Canyon Grocery"), _
        New XElement("ContactName", "Paula Wilson"), _
        New XElement("ContactTitle", "Assistant Sales Representative"), _
        New XElement("Address", "2817 Milton Dr."), _
        New XElement("City", "Albuquerque"), _
        New XElement("Region", "NM"), _
        New XElement("PostalCode", "87110"), _
        New XElement("Country", "USA"), _
        New XElement("Phone", "(505) 555-5939"), _
        New XElement("Fax", "(505) 555-3620"))

        'Define a static ShipTo group
        Dim expShipTo = _
        New XElement("ShipTo", _
        New XElement("ShipName", "Rattlesnake Canyon Grocery"), _
        New XElement("ShipAddress", "2821 Milton Dr."), _
        New XElement("ShipCity", "Albuquerque"), _
        New XElement("ShipRegion", "NM"), _
        New XElement("ShipPostalCode", "87110"), _
        New XElement("ShipCountry", "USA"))

        'Define three static LineItem groups
        Dim expLineItem1 = _
        New XElement("LineItem", _
        New XAttribute("OrderID", 10262), _
        New XElement("ProductID", 5), _
        New XElement("UnitPrice", 17.0), _
        New XElement("Quantity", 12), _
        New XElement("Discount", 0.2))

        Dim expLineItem2 = _
        New XElement("LineItem", _
        New XAttribute("OrderID", 10262), _
        New XElement("ProductID", 7), _
        New XElement("UnitPrice", 24.0), _
        New XElement("Quantity", 15), _
        New XElement("Discount", 0.0))

        Dim expLineItem3 = _
        New XElement("LineItem", _
        New XElement("Quantity", 2), _
        New XAttribute("OrderID", 10262), _
        New XElement("ProductID", 56), _
        New XElement("UnitPrice", 30.4), _
        New XElement("Discount", 0.0))

        'Create a SalesOrder parent XElement
        Dim expOrder = _
        New XElement("SalesOrder", _
        New XAttribute("OrderID", 10262), _
        New XAttribute("EmployeeID", 8), _
        New XAttribute("ShipVia", 3), _
        expBillTo, _
        New XElement("OrderDate", "1996-7-22"), _
        New XElement("RequiredDate", "1996-8-19"), _
        New XElement("ShipDate", "1996-7-25"), _
        New XElement("Freight", 48.29), _
        expShipTo, _
        New XElement("LineItems", _
        expLineItem1, expLineItem2, expLineItem3))

        'Add calculated items and order total after the </LineItems> element
        Dim decFreight As Decimal = 48.29D
        Dim xeLineItems As XElement = expOrder.Element("LineItems")
        xeLineItems.AddAfterThis(New XElement("Summary", _
        New XElement("Items", SalesOrderTotal(expOrder)), _
        New XElement("Freight", decFreight), _
        New XElement("OrderTotal", SalesOrderTotal(expOrder) + decFreight)))

        txtXML.Text = expOrder.ToString
    End Sub

    Function SalesOrderTotal(ByVal xeSO As XElement) As Decimal
        'Calculate LineItems discounted total
        Dim decTotal As Decimal
        For Each xeItem As XElement In xeSO.Descendants("LineItem")
            Dim decPrice As Decimal = CDec(xeItem.Element("UnitPrice"))
            Dim decQuantity As Integer = CInt(xeItem.Element("Quantity"))
            Dim decDiscount As Decimal = CDec(xeItem.Element("Discount"))
            decTotal += decQuantity * decPrice * (1 - decDiscount)
        Next
        Return Math.Round(decTotal, 2)
    End Function

    Private Sub SalesOrderLiteral()
        'Copy and past XML content to create Visual Basic XML literals

        Dim litBillTo = _
  <BillTo CustomerID="RATTC">
    <CompanyName>Rattlesnake Canyon Grocery</CompanyName>
    <ContactName>Paula Wilson</ContactName>
    <ContactTitle>Assistant Sales Representative</ContactTitle>
    <Address>2817 Milton Dr.</Address>
    <City>Albuquerque</City>
    <Region>NM</Region>
    <PostalCode>87110</PostalCode>
    <Country>USA</Country>
    <Phone>(505) 555-5939</Phone>
    <Fax>(505) 555-3620</Fax>
  </BillTo>

        Dim litShipTo = _
  <ShipTo>
    <ShipName>Rattlesnake Canyon Grocery</ShipName>
    <ShipAddress>2821 Milton Dr.</ShipAddress>
    <ShipCity>Albuquerque</ShipCity>
    <ShipRegion>NM</ShipRegion>
    <ShipPostalCode>87110</ShipPostalCode>
    <ShipCountry>USA</ShipCountry>
  </ShipTo>

        Dim litLineItem1 = _
    <LineItem OrderID="10262">
      <ProductID>5</ProductID>
      <UnitPrice>17</UnitPrice>
      <Quantity>12</Quantity>
      <Discount>0.2</Discount>
    </LineItem>
        Dim litLineItem2 = _
    <LineItem OrderID="10262">
      <ProductID>7</ProductID>
      <UnitPrice>24</UnitPrice>
      <Quantity>15</Quantity>
      <Discount>0</Discount>
    </LineItem>
        Dim litLineItem3 = _
    <LineItem OrderID="10262">
      <Quantity>2</Quantity>
      <ProductID>56</ProductID>
      <UnitPrice>30.4</UnitPrice>
      <Discount>0</Discount>
    </LineItem>

        Dim litOrder = _
<SalesOrder OrderID="10262" EmployeeID="8" ShipVia="3">
  <%= litBillTo %>
  <OrderDate>1996-7-22</OrderDate>
  <RequiredDate>1996-8-19</RequiredDate>
  <ShipDate>1996-7-25</ShipDate>
  <Freight>48.29</Freight>
  <%= litShipTo %>
  <LineItems>
    <%= litLineItem1 %>
    <%= litLineItem2 %>
    <%= litLineItem3 %>
  </LineItems>
</SalesOrder>

        'Add calculated items and order total after </LineItems>
        Dim xeLineItems As XElement = litOrder.Element("LineItems")
        xeLineItems.AddAfterThis(New XElement("Summary", _
        New XElement("Items", SalesOrderTotal(litOrder)), _
        New XElement("Freight", 48.29), _
        New XElement("OrderTotal", SalesOrderTotal(litOrder) + 48.29)))

        'Display the formatted document (with fix for Lf -> CrLf)
        txtXML.Text = Replace(litOrder.ToString, vbLf, vbCrLf)
        txtXML.Text = Replace(txtXML.Text, "><", ">" + vbCrLf + "<")
    End Sub

End Class
